use anyhow::{Context, Result};
use tokio::fs;

#[derive(Clone, Debug)]
pub struct SwuVersion {
    pub component: String,
    pub version: String,
}

pub async fn get_swu_versions() -> Result<Vec<SwuVersion>> {
    fs::read_to_string("/etc/sw-versions")
        .await
        .context("Could not read /etc/sw-versions")
        .map(|s| parse_swu_versions(&s))
}

/// just split by line and space
fn parse_swu_versions(content: &str) -> Vec<SwuVersion> {
    content
        .split('\n')
        .map(|s| s.trim())
        .filter_map(|s| {
            let (component, version) = s.split_once(' ')?;
            Some(SwuVersion {
                component: component.into(),
                version: version.into(),
            })
        })
        .collect()
}

#[cfg(tests)]
mod tests {
    use crate::swu::version::{parse_swu_versions, SwuVersion};

    fn test_parse_watch_urls() {
        let versions = parse_swu_versions(
            r#"test 1
 test2 2 with space"#,
        );
        assert_eq!(
            &versions,
            &[
                SwuVersion {
                    component: "test".into(),
                    version: "1".into()
                },
                SwuVersion {
                    component: "test2".into(),
                    version: "2 with space".into()
                },
            ]
        );
        let versions = parse_watch_urls("");
        assert_eq!(versions, &[]);
    }
}
